/*
 * Export a block device as a server.
 *
 * Copyright (C) 2018, 2019 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#include <stdlib.h>
#include <sys/stat.h>

#include <fsserver/resource_server.h>
#include "device.h"



/* Arguments: <ROM filename> <number of pages> */

int main(int argc, char *argv[])
{
  FILE *fp;
  size_t size;
  struct stat st;
  char *buffer;
  int num_pages;

  if (argc < 3)
  {
    printf("Need a filesystem to load and a number of pages to reserve for data.\n");
    return 1;
  }

  if (lstat(argv[1], &st))
  {
    printf("Could not get details for %s.\n", argv[1]);
    return 1;
  }

  num_pages = atoi(argv[2]);

  /* Allocate a buffer for the block device data. */

  buffer = (char *) malloc(st.st_size);
  if (buffer == NULL)
  {
    printf("Could not allocate buffer.\n");
    return 1;
  }

  fp = fopen(argv[1], "r");
  if (fp == NULL)
  {
    printf("Could not open %s.\n", argv[1]);
    return 1;
  }

  size = fread(buffer, sizeof(char), st.st_size, fp);
  fclose(fp);

  /* Provide a page collection. */

  Pages *pages = new Pages(L4_PAGESIZE * num_pages);
  Paging *paging = new Paging(pages);

  /* Initialise and register a new server object. */

  Block_device resource(paging, buffer, size);
  ResourceServer server(&resource);

  if (server.bind("block"))
  {
    printf("Could not bind thread.\n");
    return 1;
  }

  /* Enter the IPC server loop. */

  printf("Starting blockserver %p.\n", &resource);
  server.start();
  return 0;
}
